// Parameter estimates and option prices for the Duan (1995) option pricing model.
// Uses the Nelder Mead algorithm to find the maximum likelihood estimates (MLE) of the GARCH parameters.
// Uses the MLEs to simulate stock prices.  Use terminal simulated prices to price calls and puts.
// By Fabrice Douglas Rouah, 2009.
// For Visual C++.Net Version 7.1

#include "stdafx.h"
#using <mscorlib.dll>
using namespace System;
using namespace std;
#include <complex>
#include <iostream>
#include <fstream>
#include <iomanip>
#include <vector>
#include <cmath>
#include <algorithm>
#include <time.h>
#include <cstdlib>				// For srand() and rand().
#include "math.h"
#include "GARCH_LL.h"			// Contains GARCH log-likelihood function.
#include "NMfiles.h"			// Contains the Nelder-Mead algorithm files.
#include "NormalFunctions.h"	// Contains CDF and PDF for standard normal.
#include "BlackScholes.h"		// Contains Black-Scholes pricing function.
#include "VectorMean.h"			// Contains function to calculate the mean of a vector.

int main() {
	// Settings for Nelder Mead Algorithm.
	double NumIters = 1;			// First Iteration.
	double MaxIters = 1e3 ;			// Maximum number of iterations.
	double Tolerance = 1e-10;		// Tolerance on best and worst function values.
  	int N = 4;						// Number of GARCH parameters.

	// Settings for GARCH option prices.
	double S = 100.0;				// Spot Price.
	double K = 102.0;				// Strike Price.
	int T = 30;						// Maturity in days.
	double Mat = T/360.0;			// Maturity in years.
	double rf = 0.0;				// Interest Rate.
	double q  = 0.0;				// Dividend yield.
	int Nsims = 500;				// Number of simulated stock price paths.

	// Open the S&P 100 index price levels.
	ifstream inPrices;
	inPrices.open("SP100short.txt");
	vector<double> Prices(0, 0.0);
	double n1;
	int i=0;
	while (!inPrices.eof()) {
		inPrices >> n1;
		Prices.push_back(n1);
		i++ ;
	}
	int M = Prices.size();

	// Create the log returns
	vector<double> ret(M, 0.0);
	ret[0] = 0.0;
	for (int i=0; i<=M-1; i++) {
		ret[i] = log(Prices[i]/Prices[i-1]);
	}

	// GARCH parameter starting values (vertices) in vector form.
	vector<vector<double> > x(N, vector<double>(N+1));
	x[0][0]= 0.00002;	x[0][1]= 0.00001;	x[0][2]= 0.00002;	x[0][3]= 0.00001;	x[0][4]= 0.00002;	// alpha0
	x[1][0]= 0.21;		x[1][1]= 0.19;		x[1][2]= 0.16;		x[1][3]= 0.18;		x[1][4]= 0.22;		// alpha1
	x[2][0]= 0.71;		x[2][1]= 0.72;		x[2][2]= 0.69;		x[2][3]= 0.65;		x[2][4]= 0.73;		// beta1
	x[3][0]= 0.007 ;	x[3][1]= 0.008;		x[3][2]= 0.006;		x[3][3]= 0.004;		x[3][4]= 0.005;		// lambda

	// Run Nelder Mead and output Nelder Mead results.
	cout << "Nelder Mead Maximization of log=likelihood for GARCH parameters" << endl;
	vector<double> B = NelderMead(GARCH_LL, N, NumIters, MaxIters, Tolerance, x, ret, S, rf);
	double GARCHvar = B[0] / (1 - B[1]*(1+B[3]*B[3]) - B[2]);		// GARCH stationary variance
	double GARCHvol = sqrt(GARCHvar*360.0);							// GARCH stationary volatility
	cout << "-----------------------------" << endl;
	cout << "alpha0 = " << B[0] << endl;
	cout << "alpha1 = " << B[1] << endl;
	cout << "beta1  = " << B[2] << endl;
	cout << "lambda = " << B[3] << endl;
	cout << "Value of Objective Function = " << B[N] << endl;
	cout << "Number of Iterations = " << B[N+1] << endl;
	cout << "GARCH stationary volatilty = " << GARCHvol << endl;
	cout << "-----------------------------" << endl;
	cout << endl;

	// Simulate paths for the stock price, and retain the terminal prices S(T).
	vector<double> h(T, 0.0);			// GARCH variances.
	vector<double> e(T, 0.0);			// GARCH error terms.
	vector<double> SS(T, 0.0);			// Simulated stock price path S(t).
	vector<double> ST(Nsims, 0.0);		// Initialize S(T) and retain only S(T) from the S(t) paths.
	vector<double> Call(Nsims, 0.0);
	vector<double>  Put(Nsims, 0.0);
	double GARCH_call;
	double GARCH_put;
	double U;							// Random Uniform(0,1) variate.
	double Z;							// Random N(0,1) variate by inversion.
	srand (time(NULL));					// Initialize the random number seed.
	for (int i=0; i<=Nsims-1; i++) {
		SS[0] = S;						// Initialize simulated stock price to Spot.
		h[0] = GARCHvar;				// Initialize GARCH variance to stationary variance.
		e[0] = 0.0;						// Initialize error term to 0.
		for (int t=1; t<=T-1; t++) {
			U = ((double)rand() / ((double)(RAND_MAX)+(double)(1)) );
			Z = Ni(U);
			h[t] = B[0] + B[1]* pow(e[t-1] - B[3]*sqrt(h[t-1]), 2) + B[2]*h[t-1];
			e[t] = sqrt(h[t])*Z;
			SS[t] = SS[t-1]*exp(rf - 0.5*h[t] + e[t]);
		}
		Call[i] = max(0.0, SS[T-1] - K);
		Put[i]  = max(0.0, K - SS[T-1]);
	}
	// GARCH call and put price.
	GARCH_call = exp(-rf*T) * VecMean(Call);
	GARCH_put  = exp(-rf*T) * VecMean(Put);

	// Output the GARCH prices.
	cout << "GARCH call = " << GARCH_call << endl;
	cout << "GARCH put  = " << GARCH_put << endl;
	cout << "Using " << Nsims << " simulations based on " << M << " observations" << endl;
	cout << "-----------------------------" << endl;
	cout << endl;

	// Output the Black-Scholes prices
	cout << "Black-Scholes Call = " << BlackScholes(S, K, rf, q, GARCHvol, Mat, 'C') << endl;
	cout << "Black-Scholes Put  = " << BlackScholes(S, K, rf, q, GARCHvol, Mat, 'P') << endl;
	cout << "-----------------------------" << endl;
	cout << endl;
}
